#!/bin/bash
#
# stageADIC.sh
#
# This takes files in the ADIC pre-stage area, tars and gzips the data, and places the
# resulting file into and ADIC staging area where some other process writes it to tape.
#
# Terence Bullett
# CIRES
# 23Dec09
# 
# useage:   stageADIC {options}
#
# Command Line Parameters
# Optional switches
# -a {age} Archives data less than {age} days
# -d sets debug mode
# -s StationList   Processes this list of stations
#
#
# -----------------------------------------------------------------------------------------------
#
# Standard File Names:
#
# A standard individual filename is: BC840_2005285091505.MMM    (23 chars)
# A standard dayfile name is         BC840_2005285.MMM          (17 chars)
# Individual image file names        BC840_2005285091505_IO.GIF (26 chars)
# Files of name length less than 17 characters cannot be a standard filename
# Files of name length 17 or more can be handled properly.
# With standard file names, the MIDS directory structure can be determined uniquely.
#
# Tarball names are like this:
#   BC840/individual/2008/BC840_individual_2008285_2008-10-17_root.tgz        - Day of year tarball, "root" content.
#   BC840/individual/2008/BC840_individual_2008285_2008-10-17_ionogram.tgz    - Day of year tarball, "ionogram" content.
#   BC840/individual/2008/BC840_individual_2008285_2008-10-17_scaled.tgz      - Day of year tarball, "scaled" content.
#   BC840/individual/2008/BC840_individual_2008285_2008-10-17_image.tgz       - Day of year tarball, "image" content.
#   BC840/individual/2008/BC840_individual_2008285_2008-10-17_misc.tgz        - Day of year tarball, "misc" content.
#
# Requires:
#   adic_manifest -- creates an md5 manifest of the staged tarball.  It is included in this package.
#
# Version History
# 0.1  - Initial version
# 0.20 - Changed philisophy of the resultant tar files to include generation time in the filename
#        this avoids clashes and over-writes in ADIC if new data show up at a later date.
#        12Sep06 - Changed tar to use gzip.  Use rsync to move tarball into ADIC-stage
#        15Sep06 - Fixed bug when there are no ionograms or scaled data.
# 0.21 - 07Oct06 - RJR fixed bug in "find command" that was matching on other than day of year directories.
#        12Oct06 - RJR fixed bug in parsing path to station day of year that was causing this program to
#                  attempt to tar up the wrong directory.  This was due to the un-robust requirements of
#                  dates on directories matching the date of the data.  Now date/time of directories is
#                  irrelevant.
# 0.22 - 12Jan07 - RJR fixed bug where errors from rsync were being written to standard log file instead of error log file
#                  RJR add code to create a manifest file that will be stored in /nfs/iono/documentation
# 0.23 - 01Apr07 - RJR minor adustment to skip searching for non-existent stations.  This should reduce the 
#                  STDOUT/STDERR and hence number of email messages send to iono since I don't like to 
#                  redirect cronjobs to /dev/null unless I have to.
# 0.24 - 04Sep08 - RJR Minor changes, interfacing new 'tar_manifest' program
# 0.40 - 22Dec09 - TWB Updated to mirrion2 and new filing system.
#                  New logfile standards
#                  Divide into ~3 timescopes * ~5 contexts = 15 tarballs per station-day
#                  Depricate the need for ADIClist.  Act upon all stations in the Prestage buffer
#                  Added command line switches for data age, debug, etc.
#                  Incorporate adic_manifiest instead of tar_manifiest

APPNAME="stageADIC"
SRCROOT="/nfs/MIDS/tmp/ADICprestage"   
#Testing SRCROOT="/data/outgoing/test"  # Testing
# old archive, changed 23Jun11  DESTROOT="/archive/ingest/prestage/insitu_surface/ionosonde/data2" 
DESTROOT="/archive/ingest/automated/insitu_surface/ionosonde/data" 
# DESTROOT="/data/outgoing/ADICstage"    # Testing 
TMPROOT="/data/tmp/$APPNAME"
TFMT='%F %T %Z'
REALTIME=$(date  -u +"$TFMT" )
YYYY=$(date -u +%Y)
DDD=$(date -u +%j)
LOGFILE="$HOME/log/$APPNAME-$YYYY-$DDD.log"
ERRLOG="$HOME/log/$APPNAME-$YYYY-$DDD.errlog"
#
declare -i DEBUG MINAGE EPOCH ET DT D0 D1 ALLSTATION
mkdir -p $TMPROOT $DESTROOT
DEBUG=$((0))
ALLSTATION=$((1))

# Minimum age of data to consider for ADIC staging 
MINAGE=$((30))
#
# Current Epoch Time
let EPOCH=$(date +%s )

while getopts 'a:dhs:' OPTION
do
   case $OPTION in
      a) MINAGE=$((OPTARG))
         ;;
      d) DEBUG=$((1))
         ;;
      h) 
         echo "stageADIC.sh Version 0.40"
         echo "Optional switches"
         echo "-a {age} Sets minimum data age, in days"   
         echo "-d sets debug mode"
         echo "-h help"
	 echo "-s {station list} process select stations"
         exit 0
      ;;
      s) STATIONLIST="$OPTARG"
         ALLSTATION=$((0)) 
      ;;
   esac
done
# Remove the switches from the list.  Should leave only the file name to buffer and the destination list
shift $((OPTIND - 1))
#
#
# Remove previous empty directories
find $DESTROOT/ -type d -empty -exec rmdir {} \; > /dev/null  2>&1 
find $SRCROOT/ -type d -empty -exec rmdir {} \; > /dev/null  2>&1 
#
# Turn minumum age into seconds
MINAGE=$((MINAGE*24*3600))
DISTLIST=$(ls "$SRCROOT" )
for DIST in $DISTLIST ; do        # Distributions, e.g. public or private
   if (($ALLSTATION)) ; then 
      STATIONLIST=$(ls "$SRCROOT/$DIST" )
   fi
#db   echo $STATIONLIST >> $ERRLOG
   for STATION in $STATIONLIST ; do
      SCOPELIST=$(ls  "$SRCROOT/$DIST/$STATION" 2>/dev/null)
      for TIMESCOPE in $SCOPELIST ; do
         YEARLIST=$(ls  "$SRCROOT/$DIST/$STATION/$TIMESCOPE" 2>/dev/null)
         for YEAR in $YEARLIST ; do 
            if (($DEBUG)) ; then 
              echo "$REALTIME - $APPNAME - DEBUG - Release $DIST from $STATION Timescope $TIMESCOPE Year $YEAR Age $MINAGE"  >> $LOGFILE
            fi
            if [ $TIMESCOPE = 'individual' ] ; then
               DAYLIST=$(ls "$SRCROOT/$DIST/$STATION/$TIMESCOPE/$YEAR" 2>/dev/null)
               for DAY in $DAYLIST ; do
	       	  REALTIME=$(date  -u +"$TFMT" )
#                 bash math does not like leading zeros
                  let D0=$(echo $DAY | sed 's/0*//') 2>>$ERRLOG 
                  D1=$((D0-1))
                  let ET=$(date +%s --date "01 Jan $YEAR $D1 days")
                  DT=$((EPOCH-ET))
#db                  echo $DIST $STATION $TIMESCOPE >> $ERRLOG 
#db		  echo $STATION $YEAR : 'Day= '$DAY' D0= '$D0' D1= '$D1' ET= '$ET' DT= '$DT  >> $ERRLOG
                  if [ $DT -gt $MINAGE ] ; then
#                    Stage these data
                     echo "$REALTIME - $APPNAME - INFO - Staging  $DIST from $STATION Timescope $TIMESCOPE date $YEAR-$DAY "  >> $LOGFILE
#                    Public/Private asymmetry
                     if [ $DIST != 'public' ] ; then 
                        DESTINATION="$DESTROOT/nonpublic/$STATION/$TIMESCOPE/$YEAR/$DAY"
                     else
                        DESTINATION="$DESTROOT/$STATION/$TIMESCOPE/$YEAR/$DAY"
                     fi
#
                     CONTEXTLIST=$(ls -p "$SRCROOT/$DIST/$STATION/$TIMESCOPE/$YEAR/$DAY" | grep / )
                     for CONTEXT in $CONTEXTLIST ; do
#                       Strip the trailing /
                        CONTEXT=${CONTEXT%%/}
                        SRCDIR="$DIST/$STATION/$TIMESCOPE/$YEAR/$DAY/$CONTEXT"
                        DATE=$(date +%Y-%m-%d)
                        TARBALL=$STATION'_'$TIMESCOPE'_'$YEAR$DAY'_'$DATE'_'$CONTEXT'.tar.gz'                        
                        OUTFILE="$TMPROOT/$TARBALL"
                        if (( $DEBUG )) ; then
                           echo "$REALTIME - $APPNAME - DEBUG - SourceDir: $SRCROOT/$SRCDIR"     >> $LOGFILE 
                           echo "$REALTIME - $APPNAME - DEBUG - Outfile: $OUTFILE"   >> $LOGFILE
                           echo "$REALTIME - $APPNAME - DEBUG - Destination: $DESTINATION"  >> $LOGFILE
                           tar -cvzf $OUTFILE -C $SRCROOT/ ./$SRCDIR >> $ERRLOG 2>&1
                        else
                           tar -czf $OUTFILE -C $SRCROOT/ ./$SRCDIR >> $ERRLOG 2>&1
                        fi
                        echo "adic_manifest $OUTFILE  $SRCROOT/$SRCDIR/"  >>$ERRLOG
                        adic_manifest "$OUTFILE"  "$SRCROOT/$SRCDIR/"  >> $LOGFILE 2>>$ERRLOG 
                        mkdir -p $DESTINATION  2>>$ERRLOG
#                       mv $OUTFILE $DESTINATION/  2>>$ERRLOG
#debug                  cp $OUTFILE $DESTINATION/  2>>$ERRLOG			
			rsync -W $OUTFILE $DESTINATION 2>>$ERRLOG 
			EXS=$?
			if [ $EXS != 0 ] ; then
			   echo "$REALTIME - $APPNAME - ERROR - rsync of $OUTFILE failed status $EXS"     >> $LOGFILE
			else
#                          Get rid of the source data
			   rm -f "$OUTFILE" 2>$ERRLOG
			   rm -rf "$SRCROOT/$SRCDIR"   2>>$ERRLOG
			fi
                     done  # Contexts 
#                    Take care of the (probably empty) root directory
#                    Not implemented
                  else
#                    Data are not old enough
                     if (( $DEBUG )) ; then 
                        echo "$REALTIME - $APPNAME - DEBUG - Not staging  $DIST from $STATION Timescope $TIMESCOPE date $YEAR-$DAY "  >> $LOGFILE
                     fi
                  fi 
               done  # Days
#              Remove empty directories
               find $SRCROOT/ -type d -empty -exec rmdir {} \; > /dev/null  2>&1  
            else
               echo "$REALTIME - $APPNAME - ERROR - Timescope $TIMESCOPE is not supported" >> $LOGFILE
               exit -2
            fi # Known Timescopes
#           Gently Remove empty directories
            find $SRCROOT/ -type d -empty -exec rmdir {} \; > /dev/null 2>&1  
         done  # Years
#        Gently Remove empty directories
         find $SRCROOT/ -type d -empty -exec rmdir {} \; > /dev/null  2>&1   
      done     # Timescopes
#     Gently Remove empty directories
      find $SRCROOT/ -type d -empty -exec rmdir {} \; > /dev/null 2>&1  
   done        # Stations
#  Gently Remove empty directories
   find $SRCROOT/ -type d -empty -exec rmdir {} \; > /dev/null 2>&1  
done     # Releases
 
# Hack to remove data before 01Jan2010
# TWB  23Jun11 find /archive/ingest/prestage/insitu_surface/ionosonde/data2/ -iname '*_200????_*' -exec rm -f {} \;   2>>$ERRLOG 

exit 0 
